"use strict";
// Copyright (c) 2024, Brandon Lehmann <brandonlehmann@gmail.com>
//
// Permission is hereby granted, free of charge, to any person obtaining a copy
// of this software and associated documentation files (the "Software"), to deal
// in the Software without restriction, including without limitation the rights
// to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
// copies of the Software, and to permit persons to whom the Software is
// furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
// AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
// OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
// SOFTWARE.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.WebSocket = exports.Buffer = void 0;
const events_1 = require("events");
const buffer_1 = require("buffer");
Object.defineProperty(exports, "Buffer", { enumerable: true, get: function () { return buffer_1.Buffer; } });
const timer_1 = __importDefault(require("@gibme/timer"));
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
var WebSocketState;
(function (WebSocketState) {
    WebSocketState[WebSocketState["CONNECTING"] = 0] = "CONNECTING";
    WebSocketState[WebSocketState["OPEN"] = 1] = "OPEN";
    WebSocketState[WebSocketState["CLOSING"] = 2] = "CLOSING";
    WebSocketState[WebSocketState["CLOSED"] = 3] = "CLOSED";
})(WebSocketState || (WebSocketState = {}));
class WebSocket extends events_1.EventEmitter {
    /**
     * Constructs a new instances of the class
     * @param options
     */
    constructor(options) {
        var _a;
        var _b;
        super();
        this.options = options;
        this.pre_ready_sent_messages = [];
        (_a = (_b = this.options).binaryType) !== null && _a !== void 0 ? _a : (_b.binaryType = 'arraybuffer');
    }
    /**
     * Returns the underlying WebSocket's binary type
     */
    get binaryType() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.binaryType) || 'arraybuffer';
    }
    /**
     * Sets the underlying WebSocket's binary type
     *
     * @param value
     */
    set binaryType(value) {
        if (this.socket && this.socket.binaryType) {
            this.socket.binaryType = value;
        }
    }
    /**
     * Returns the amount of data buffered by the underlying WebSocket
     */
    get bufferedAmount() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.bufferedAmount) || 0;
    }
    /**
     * Returns any extensions currently enabled in the underlying WebSocket
     */
    get extensions() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.extensions) || '';
    }
    /**
     * Returns the underlying WebSocket protocol
     */
    get protocol() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.protocol) || '';
    }
    /**
     * Returns the underlying WebSocket's ready state
     */
    get readyState() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.readyState) || 3;
    }
    /**
     * Returns the underlying WebSocket's remote URL
     */
    get url() {
        var _a;
        return ((_a = this.socket) === null || _a === void 0 ? void 0 : _a.url) || this.options.url;
    }
    /** @ignore */
    on(event, listener) {
        return super.on(event, listener);
    }
    /** @ignore */
    once(event, listener) {
        return super.on(event, listener);
    }
    /** @ignore */
    off(event, listener) {
        return super.on(event, listener);
    }
    /**
     * Attempts to connect the underlying WebSocket to the remote URL
     *
     * This method also allows us to re-use the class instance, so we do not have
     * to recreate it if we need to 'reconnect' the WebSocket later
     */
    open() {
        try {
            this.socket = new isomorphic_ws_1.default(this.options.url, this.options.protocols);
            this.binaryType = this.options.binaryType || 'arraybuffer';
            this.timer = new timer_1.default(10, false);
            this.timer.on('tick', () => {
                var _a;
                if (this.readyState === WebSocketState.OPEN) {
                    (_a = this.timer) === null || _a === void 0 ? void 0 : _a.destroy();
                    this.emit('ready');
                    // if we tried to send messages before we were ready, send them now
                    const messages = this.pre_ready_sent_messages.slice(0);
                    this.pre_ready_sent_messages = [];
                    for (const message of messages) {
                        this.send(message);
                    }
                }
            });
            this.socket.onopen = () => {
                var _a;
                this.emit('open');
                (_a = this.timer) === null || _a === void 0 ? void 0 : _a.start();
                if (this.socket) {
                    this.socket.onclose = () => {
                        this.emit('close');
                        if (this.options.autoReconnect) {
                            this.open();
                        }
                    };
                }
            };
            this.socket.onclose = () => {
                if (this.options.autoReconnect) {
                    this.open();
                }
            };
            this.socket.onerror = error => this.emit('error', new Error(error.toString()));
            this.socket.onmessage = event => {
                if (event.data instanceof ArrayBuffer) {
                    return this.emit('message', buffer_1.Buffer.from(event.data));
                }
                else if (event.data instanceof buffer_1.Buffer) {
                    return this.emit('message', event.data);
                }
                else if (Array.isArray(event.data)) {
                    if (event.data instanceof buffer_1.Buffer) {
                        event.data.forEach(item => {
                            this.emit('message', item);
                        });
                    }
                }
                else {
                    return this.emit('message', buffer_1.Buffer.from(event.data));
                }
            };
        }
        catch (error) {
            delete this.socket;
            this.emit('error', new Error(error.toString()));
        }
    }
    /**
     * Alias for `open()`
     */
    connect() {
        return this.open();
    }
    /**
     * Closes the underlying WebSocket
     *
     * @param code
     * @param reason
     */
    close(code, reason) {
        if (this.socket) {
            // stomp over the auto-reconnect if we told it to close
            this.socket.onclose = () => this.emit('close');
            this.socket.close(code, reason);
        }
    }
    /**
     * Sends the specified data via the underlying WebSocket
     *
     * @param data
     */
    send(data) {
        var _a;
        // if we try to send before we are ready, stack them up
        if (this.readyState !== WebSocketState.OPEN) {
            this.pre_ready_sent_messages.push(data);
        }
        else {
            (_a = this.socket) === null || _a === void 0 ? void 0 : _a.send(data);
        }
    }
}
exports.WebSocket = WebSocket;
exports.default = WebSocket;
//# sourceMappingURL=index.js.map